@@ -139,6 +139,23 @@ public:
     }
 };
 
+static QString get_resource(QString file)
+{
+    QFileInfo appDir(qApp->applicationDirPath());
+
+    // first search in the directory containing dvbcut
+    QString resFile = appDir.absoluteFilePath() + file;
+
+  #ifndef __WIN32__
+    // Unix/Linux: search in the associated share subdirectory
+    if (!QFile::exists(resFile)) {
+      resFile = appDir.absolutePath() + "/share/dvbcut" + file;
+    }
+  #endif
+
+    return resFile;
+}
+
 // **************************************************************************
 // ***  dvbcut::dvbcut (private constructor)
 
@@ -196,6 +213,7 @@ dvbcut::dvbcut()
 
   // set caption
   setWindowTitle(QString(VERSION_STRING));
+  setWindowIcon(QIcon(get_resource("/icons/dvbcut.svg")));
 }
 
 // **************************************************************************
@@ -562,7 +580,7 @@ void dvbcut::fileExport()
     }
   }
 
-  std::auto_ptr<exportdialog> expd(new exportdialog(QString::fromStdString(expfilen),this));
+  std::unique_ptr<exportdialog> expd(new exportdialog(QString::fromStdString(expfilen),this));
   expd->ui->muxercombo->addItem(tr("MPEG program stream/DVD (DVBCUT multiplexer)"));
   expd->ui->muxercombo->addItem(tr("MPEG program stream (DVBCUT multiplexer)"));
   expd->ui->muxercombo->addItem(tr("MPEG program stream/DVD (libavformat)"));
@@ -707,7 +725,7 @@ void dvbcut::fileExport()
 
   //   lavfmuxer mux(fmt,*mpg,outfilename);
 
-  std::auto_ptr<muxer> mux;
+  std::unique_ptr<muxer> mux;
   uint32_t audiostreammask(0);
 
   for(int a=0;a<mpg->getaudiostreams();++a)
@@ -719,17 +737,17 @@ void dvbcut::fileExport()
 
   switch(expfmt) {
     case 1:
-      mux=std::auto_ptr<muxer>(new mpegmuxer(audiostreammask,*mpg,out_file.c_str(),false,0));
+      mux = std::unique_ptr<muxer>(new mpegmuxer(audiostreammask,*mpg,out_file.c_str(),false,0));
       break;
     case 2:
-      mux=std::auto_ptr<muxer>(new lavfmuxer("dvd",audiostreammask,*mpg,out_file.c_str()));
+      mux = std::unique_ptr<muxer>(new lavfmuxer("dvd",audiostreammask,*mpg,out_file.c_str()));
       break;
     case 3:
-      mux=std::auto_ptr<muxer>(new lavfmuxer("mpegts",audiostreammask,*mpg,out_file.c_str()));
+      mux = std::unique_ptr<muxer>(new lavfmuxer("mpegts",audiostreammask,*mpg,out_file.c_str()));
       break;
     case 0:
     default:
-      mux=std::auto_ptr<muxer>(new mpegmuxer(audiostreammask,*mpg,out_file.c_str()));
+      mux = std::unique_ptr<muxer>(new mpegmuxer(audiostreammask,*mpg,out_file.c_str()));
       break;
   }
 
@@ -1115,6 +1133,87 @@ void dvbcut::viewDifference()
   updateimagedisplay();
 }
 
+inline int square(int x)
+{
+    return x * x;
+}
+
+unsigned long calc_distance(QImage baseimg, QImage im)
+{
+    if (im.size() != baseimg.size())
+        im = im.scaled(baseimg.size());
+
+    unsigned long distance = 0;
+    if (im.depth() == 32 && baseimg.depth() == 32) {
+        for (int y = 0; y < baseimg.height(); ++y) {
+            QRgb *imd = (QRgb*) im.scanLine(y);
+            QRgb *bimd = (QRgb*) baseimg.scanLine(y);
+
+            for (int x = im.width(); x > 0; --x) {
+                int dist = square(qRed(*imd) - qRed(*bimd)) + square(qGreen(*imd) - qGreen(*bimd)) + square(qBlue(*imd) - qBlue(*bimd));
+                if (dist > 1000)
+                    dist = 1000;
+
+                distance += dist;
+                ++imd;
+                ++bimd;
+            }
+        }
+    }
+
+    return distance;
+}
+
+int dvbcut::getPreviousStopEvent(int picture)
+{
+    int previous = 0;
+
+    for (int i = 0; i < ui->eventlist->count(); i++) {
+        EventListItem *eli = dynamic_cast<EventListItem *>(ui->eventlist->item(i));
+        if (eli->geteventtype() == EventListItem::stop && eli->getpicture() < picture)
+            previous = eli->getpicture();
+    }
+
+    if (previous)
+        return previous;
+    else
+        critical(tr("Searching ..."), tr("Please set a stop marker for the picture to be searched.\nThen go where the picture is expected to appear the next time e.g. the end of the commercial break."));
+
+    return picture;
+}
+
+void dvbcut::searchDuplicate()
+{
+    ui->searchDuplicateAction->setEnabled(false);
+
+    progressstatusbar psb(statusBar());
+    psb.print(tr("Searching ..."));
+    psb.setprogress(0);
+    imageprovider imgp(*mpg);
+
+    int search_start_pic = curpic;
+    QImage last_stop_img = imgp.getimage(getPreviousStopEvent(search_start_pic));
+
+    unsigned long min = calc_distance(last_stop_img, imgp.getimage(search_start_pic, true));
+    static const long n = settings().search_dups_range;
+
+    for (long i = 0; i < n; ++i) {
+        unsigned long d = calc_distance(last_stop_img, imgp.getimage(search_start_pic + i, true));
+        if (d < min) {
+            min = d;
+            gotoFrame(search_start_pic + i);
+        }
+
+        psb.setprogress(i * 1000 / n);
+        if (psb.cancelled()) {
+            // FIXME
+            break;
+        }
+    }
+
+    ui->searchDuplicateAction->setEnabled(true);
+}
+
 
 void dvbcut::viewUnscaled()
 {
@@ -1232,8 +1331,8 @@ void dvbcut::playPlay()
   arguments << "-geometry" << QString().sprintf("%dx%d+0+0",int(ui->imagedisplay->width()),int(ui->imagedisplay->height()));
 
   if (currentaudiotrack>=0 && currentaudiotrack<mpg->getaudiostreams()) {
-    arguments << "-aid" << QString().sprintf("0x%x",int(currentaudiotrack));
-    }
+    arguments << "-aid" << QString().sprintf("0x%x", int(mpg->mplayeraudioid(currentaudiotrack)));
+  }
     
   // for now, pass all filenames from the current one up to the last one
   std::list<std::string>::const_iterator it = mpgfilen.begin();
@@ -1292,14 +1391,22 @@ void dvbcut::playAudio2()
 
 void dvbcut::linslidervalue(int newpic)
 {
+  ui->linslider->setTracking(false);
+
   if (!mpg || newpic==curpic)
+  {
+    ui->linslider->setTracking(true);
     return;
+  }
   if (!fine)
     newpic=mpg->lastiframe(newpic);
   if (!jogsliding)
     jogmiddlepic=newpic;
   if (newpic==curpic)
+  {
+    ui->linslider->setTracking(true);
     return;
+  }
 
   curpic=newpic;
   if (!jogsliding)
@@ -1307,6 +1414,8 @@ void dvbcut::linslidervalue(int newpic)
   
   update_time_display();
   updateimagedisplay();
+
+  ui->linslider->setTracking(true);
 }
   
 void dvbcut::jogsliderreleased()
@@ -1318,8 +1427,13 @@ void dvbcut::jogsliderreleased()
 
 void dvbcut::jogslidervalue(int v)
 {
+  ui->jogslider->setTracking(false);
+
   if (!mpg || (v==0 && curpic==jogmiddlepic))
+  {
+    ui->jogslider->setTracking(true);
     return;
+  }
   jogsliding=true;
 
   int relpic=0;
@@ -1362,7 +1476,9 @@ void dvbcut::jogslidervalue(int v)
   if (curpic!=newpic)
     ui->linslider->setValue(newpic);
 
-    fine=false;
+  fine=false;
+
+  ui->jogslider->setTracking(true);
 }
 
 void dvbcut::doubleclickedeventlist(QListWidgetItem *lbi)
@@ -2564,23 +2680,11 @@ private:
   QPushButton *prev, *next, *home, *close;
 };
 
-QString get_help_file(QString locale)
+static QString get_help_file(QString locale)
 {
-  QString file = QString("/dvbcut_%1.html").arg(locale);
+    QString file = QString("/dvbcut_%1.html").arg(locale);
 
-  QFileInfo appDir(qApp->applicationDirPath());
-
-  // first search in the directory containing dvbcut
-  QString helpFile = appDir.absoluteFilePath() + file;
-
-#ifndef __WIN32__
-  // Unix/Linux: search in the associated share subdirectory
-  if (!QFile::exists(helpFile)) {
-    helpFile = appDir.absolutePath() + "/share/dvbcut" + file;
-  }
-#endif
-
-  return helpFile;
+    return get_resource(file);
 }
 
 void dvbcut::helpContentAction_activated()
@@ -2604,3 +2708,11 @@ void dvbcut::helpContentAction_activated()
       tr("Help file %1 not available").arg(helpFile));
   }
 }
+
+void dvbcut::gotoFrame(int frameno)
+{
+  bool save = fine;
+  fine = true;
+  ui->linslider->setValue(frameno);
+  fine = save;
+}
